#ifndef LOW_LEVEL_PLATFORM_H_
#define LOW_LEVEL_PLATFORM_H_

#include <Core/Base/NoInstanceBase.h>
#include <HAL/Drivers/XMC/TXmc1100DigitalOutputDriver.h>
#include <HAL/Drivers/XMC/TXmc1100SystemTickDriver.h>
#include <HAL/Drivers/XMC/TXmc1100UartDriver.h>
#include <HAL/Drivers/XMC/TXmc1100PlatformUtils.h>

/**
 * This class provides a low layer hardware abstraction, making the
 * layers above (high level drivers and application) independent of the
 * platform they are running on.
 * It defines the low level drivers for the simulation environment.
 */
template<unsigned int T = 0>
class TLowLevelPlatform: public redBlocks::Core::NoInstanceBase
{
  public:

    typedef redBlocks::HAL::Drivers::TXmc1100DigitalOutputDriver<PORT1_BASE, 0> Led;
    typedef redBlocks::HAL::Drivers::TXmc1100DigitalOutputDriver<PORT1_BASE, 1> AwakeLed;

    typedef redBlocks::HAL::Drivers::TXmc1100SystemTickDriver<> SystemTickDriver;
    typedef redBlocks::HAL::Drivers::TXmc1100UartDriver<> TerminalComInterface;

    typedef redBlocks::HAL::Drivers::Xmc1100IsrLock IsrLock;
    typedef redBlocks::HAL::Drivers::Xmc1100PowerSaveCallback PowerSaveCallback;

    /**
     * This is how we enter the sleep mode on our platform
     */
    static void enterSleepMode()
    {
      redBlocks::HAL::Drivers::Xmc1100SleepMode::enterSleepMode();
    }

    static void initLowLevelDrivers()
    {
      // configure 1000 ticks per second and enable the system tick ISR
      SystemTickDriver::configure( 1000 );
      SystemTickDriver::enableCallback();

      // configure the UART for 19200 baud (no parity) and enable the
      // receive ISR callback
      TerminalComInterface::configure( 19200 );

      // configure the pins, the LEDs are connected to as outputs
      Led::configure();
      AwakeLed::configure();
    }
};

typedef TLowLevelPlatform<> LowLevelPlatform;

#endif
